import React, { forwardRef, useCallback, useImperativeHandle, useMemo, useRef } from 'react';
import { Provider } from 'react-redux';
import EditorDrawer from "./EditorDrawer";
import Field from "./Field";
import CpuEditorContext from "./context";
import { CpuInteraction } from "./context/interaction";
import { CpuResources } from "./context/resources";
import defaultAjvInstance from "./definition/ajvInstance";
export var InfoContext = /*#__PURE__*/React.createContext(null);
var immutableEmptyArray = [];
var immutableEmptyObject = {};
var Editor = function Editor(props, ref) {
  var schema = props.schema,
    data = props.data,
    onChange = props.onChange,
    id = props.id,
    _props$viewsMap = props.viewsMap,
    viewsMap = _props$viewsMap === void 0 ? immutableEmptyObject : _props$viewsMap,
    componentMap = props.componentMap,
    rootMenuItems = props.rootMenuItems,
    _props$providerProps = props.providerProps,
    providerProps = _props$providerProps === void 0 ? immutableEmptyObject : _props$providerProps,
    _props$options = props.options,
    _props$options2 = _props$options === void 0 ? {} : _props$options,
    _props$options2$ajvIn = _props$options2.ajvInstance,
    ajvInstance = _props$options2$ajvIn === void 0 ? defaultAjvInstance : _props$options2$ajvIn,
    _props$options2$resou = _props$options2.resources,
    resourceOptions = _props$options2$resou === void 0 ? immutableEmptyObject : _props$options2$resou;

  // 详细抽屉功能
  var drawerRef = useRef(null);
  var setDrawer = useCallback(function () {
    var _drawerRef$current;
    // console.log('setDrawer', drawerRef.current);
    if (drawerRef.current) (_drawerRef$current = drawerRef.current).setDrawer.apply(_drawerRef$current, arguments);
  }, [drawerRef]);
  var interaction = useMemo(function () {
    return new CpuInteraction(setDrawer);
  }, [setDrawer]);
  var resources = useMemo(function () {
    var _resourceOptions$mapT = resourceOptions.mapToSrc,
      mapToSrc = _resourceOptions$mapT === void 0 ? function (src) {
        return src;
      } : _resourceOptions$mapT;
    return new CpuResources(mapToSrc);
  }, [resourceOptions]);

  // 新建 ctx
  var ctx = useMemo(function () {
    return new CpuEditorContext(data, schema, ajvInstance, id, interaction, resources, componentMap, viewsMap);
  }, [schema, interaction, resources, componentMap, viewsMap]);

  // ctx 变化生命周期维护(用 useMemo 而不是 useEffect，是为了一定保证 ctx 和 subscribe 变化的同时性)
  var unsubscribeChangeRef = useRef();
  useMemo(function () {
    var change = function change() {
      var changedData = ctx.store.getState().present.data;
      if (onChange && typeof onChange === 'function') {
        onChange(changedData);
      }
    };
    // 订阅 store 的 change
    if (unsubscribeChangeRef.current) unsubscribeChangeRef.current();
    var unsubscribe = ctx.store.subscribe(change);
    unsubscribeChangeRef.current = unsubscribe;
  }, [onChange, ctx]);

  // 如果 data 更新来自外部，通过 setData 与 store 同步
  var presentData = ctx.store.getState().present.data;
  if (data !== presentData) {
    // console.log('检测到外部更新：', data, presentData, ctx.schemaId)
    ctx.store.dispatch({
      type: 'setData',
      value: data
    });
  }

  // 暴露一下 api
  useImperativeHandle(ref, function () {
    return ctx;
  }, [ctx]);
  var SchemaErrorLogger = ctx.getComponent(null, ['schemaErrorLogger']);
  var GlobalProvider = ctx.getComponent(null, ['globalProvider']);
  return /*#__PURE__*/React.createElement(Provider, {
    store: ctx.store
  }, ctx.schemaError ? /*#__PURE__*/React.createElement(SchemaErrorLogger, {
    error: ctx.schemaError.toString()
  }) : null, /*#__PURE__*/React.createElement(InfoContext.Provider, {
    value: ctx
  }, /*#__PURE__*/React.createElement(GlobalProvider, providerProps, /*#__PURE__*/React.createElement(Field, {
    viewport: "window",
    route: immutableEmptyArray,
    schemaEntry: "#",
    rootMenuItems: rootMenuItems
  }), /*#__PURE__*/React.createElement(EditorDrawer, {
    ref: drawerRef
  }))));
};
export default /*#__PURE__*/React.memo( /*#__PURE__*/forwardRef(Editor));