import CpuEditorContext, { SchemaArrayRefInfo } from '.';
import { JSONSchema } from '../type/Schema';
import { virtualSchemaProps } from './virtual';
declare type processedSchemaProps = {
    type?: string[];
    properties?: {
        [k: string]: string;
    };
    patternProperties?: {
        [k: string]: string;
    };
    dependencies?: {
        [k: string]: string | string[];
    };
    /**
     * `schema`处理为 ref，`schema[]`处理为`SchemaArrayRefInfo`
     *
     * 不允许处理为`false`，不存在为`undefined`
     */
    prefixItems?: SchemaArrayRefInfo;
    items?: string | false;
    additionalProperties?: string | false;
    oneOf?: SchemaArrayRefInfo;
    anyOf?: SchemaArrayRefInfo;
};
export declare type MergedSchemaWithoutVirtual = Omit<JSONSchema, keyof processedSchemaProps | 'additionalItems'> & processedSchemaProps;
/**
 * 合并后的 schema 只涉及到了用到的这一层的信息，不对子层的信息进行进一步归纳。
 */
export declare type MergedSchema = MergedSchemaWithoutVirtual & virtualSchemaProps;
/**
 * 将 schemaMap 以合并的方式得到其中的参数
 * 1. first 还需要知道使用的 ref 在哪
 * 2. 对于对象或者数组的，值为 schema 的属性，需要一个子info来处理，也合到里面
 * @param ctx
 * @param map
 * @returns
 */
export declare const mergeSchemaMap: (ctx: CpuEditorContext, map: Map<string, JSONSchema | boolean>) => false | MergedSchema;
export {};
