function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
import _ from 'lodash';
import { toOfName } from "../definition";
import { shallowValidate } from "../definition/shallowValidate";
import { addRef, deepReplace } from "../utils";
/**
 * 验证数据符合 oneOf/anyOf 的哪一个选项
 * @param data
 * @param schemaEntry
 * @param ctx
 * @returns `null`为无 oneOf/anyOf，`false`为不符合任何选项，`string`为选项链
 */
export var getOfOption = function getOfOption(data, schemaEntry, ctx) {
  var ofCacheValue = schemaEntry ? ctx.getOfInfo(schemaEntry) : null;
  if (ofCacheValue) {
    var subOfRefs = ofCacheValue.subOfRefs,
      ofLength = ofCacheValue.ofLength,
      ofRef = ofCacheValue.ofRef;
    for (var i = 0; i < ofLength; i++) {
      var subOfRef = subOfRefs[i];
      if (typeof subOfRef === 'string') {
        // 展开的 validate 为 string，就是子 oneOf 的 ref
        var subOption = getOfOption(data, subOfRef, ctx);
        console.assert(subOption !== null);
        if (subOption) return "".concat(i, "-").concat(subOption);
      } else {
        var valid = shallowValidate(data, addRef(ofRef, i.toString()), ctx);
        if (valid) return i.toString();
      }
    }
    return false;
  }
  return null;
};

/**
 * 通过 of 链找到 schema 经层层选择之后引用的 valueEntry
 * @param ctx
 * @param schemaEntry
 * @param ofChain
 */
export var getRefByOfChain = function getRefByOfChain(ctx, schemaEntry, ofChain) {
  var ofSelection = ofChain.split('-');
  var entry = schemaEntry;
  var _iterator = _createForOfIteratorHelper(ofSelection),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var opt = _step.value;
      var _ref = ctx.getOfInfo(entry),
        ofRef = _ref.ofRef;
      entry = addRef(ofRef, opt);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return entry;
};

/**
 * 对 `schemaEntry` 设置 ofInfo
 * @param ctx
 * @param schemaEntry
 * @param rootSchema
 * @param nowOfRefs
 * @returns
 */
export var setOfInfo = function setOfInfo(ctx, schemaEntry, rootSchema) {
  var nowOfRefs = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
  var mergedSchema = ctx.getMergedSchema(schemaEntry);
  if (!mergedSchema) return null;
  // todo: noAnyOfChoice 的情况下
  var SchemaArrayRefInfo = mergedSchema.oneOf || mergedSchema.anyOf;
  if (!SchemaArrayRefInfo) return null;
  var ofRef = SchemaArrayRefInfo.ref,
    ofLength = SchemaArrayRefInfo.length;
  // 设置 ofCache (use Entry map ,root)
  if (ofRef && nowOfRefs.includes(ofRef)) {
    console.error('你进行了oneOf/anyOf的循环引用，这会造成无限递归，危', nowOfRefs, ofRef);
    ctx.ofInfoMap.set(schemaEntry, null);
    return null;
  } else if (ofRef) {
    nowOfRefs.push(ofRef);

    // 接下来得到每个选项的 ref 和树选择需要的选项 options
    var subOfRefs = [];
    var oneOfOptions = [];
    var _loop = function _loop(i) {
      var ref = addRef(ofRef, i.toString());
      var optMergedSchema = ctx.getMergedSchema(ref);
      var name = optMergedSchema ? toOfName(optMergedSchema) : '';
      var optOption = {
        value: i.toString(),
        title: name ? name : "Option ".concat(i + 1)
      };
      var optCache = ctx.ofInfoMap.has(ref) ? ctx.ofInfoMap.get(ref) : setOfInfo(ctx, ref, rootSchema, nowOfRefs);
      if (optCache) {
        var options = optCache.options;
        optOption.children = options.map(function (option) {
          return deepReplace(_.cloneDeep(option), 'value', function (prev) {
            return "".concat(i, "-").concat(prev);
          });
        });
        optOption.disabled = true;
        // 选项有子选项，将子选项ref给他
        subOfRefs.push(ref);
      } else {
        subOfRefs.push(undefined);
      }
      oneOfOptions.push(optOption);
    };
    for (var i = 0; i < ofLength; i++) {
      _loop(i);
    }
    var ofInfo = {
      subOfRefs: subOfRefs,
      ofRef: ofRef,
      ofLength: ofLength,
      options: oneOfOptions
    };
    ctx.ofInfoMap.set(schemaEntry, ofInfo);
    return ofInfo;
  } else {
    ctx.ofInfoMap.set(schemaEntry, null);
    return null;
  }
};