function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
/* eslint-disable no-param-reassign */
/* eslint-disable @typescript-eslint/no-shadow */

import localize from 'ajv-i18n/localize/zh';
import { produce } from 'immer';
import cloneDeep from 'lodash/cloneDeep';
import remove from 'lodash/remove';
import undoable from 'redux-undo';
import { processValidateErrors } from "../helper/validate-errors/processValidateErrors";
export var doAction = function doAction(type, options) {
  var schemaEntry = options.schemaEntry,
    route = options.route,
    field = options.field,
    value = options.value;
  return {
    schemaEntry: schemaEntry,
    type: type,
    route: route,
    field: field,
    value: value
  };
};
export var JsonTypes = ['object', 'array', 'string', 'number', 'boolean', 'null'];

/**
 * 导出 editor 的 reducer 方法，其中可使用 ctx 闭包上下文
 * @param ctx
 * @returns
 */
export var getReducer = function getReducer(ctx) {
  return undoable(produce(function (s, a) {
    var type = a.type,
      route = a.route,
      field = a.field,
      value = a.value,
      schemaEntry = a.schemaEntry;
    var reValidate = function reValidate(fieldValue) {
      var dataPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
      // todo: fieldModel 未存在时暂时的解法，改为根节点验证
      if (schemaEntry === undefined) {
        fieldValue = s.data;
        dataPath = [];
      }
      var uri = ctx.schemaId + (schemaEntry || '');
      var validate = ctx.ajvInstance.getSchema(uri);
      if (typeof validate === 'function') {
        console.time('ajv 验证');
        validate(fieldValue);
        localize(validate.errors);

        // process errors into Instance path
        s.dataErrors = processValidateErrors(s.dataErrors, validate.errors || [], dataPath, schemaEntry);

        // if (validate.errors) {
        //   console.warn('not valid', validate.errors)
        // }
        console.timeEnd('ajv 验证');
      }
    };

    // 特殊接口：setData 强制设置数据，且不深拷贝
    if (type === 'setData') {
      s.data = value;
      // revalidate
      reValidate(s.data);
      return;
    }

    // 初始化
    if (!route) {
      // console.log('初始化验证', a);
      reValidate(s.data);
      return;
    }
    var data = s.data; // 注意这个变量一直是 s 子节点的一个引用
    var _iterator = _createForOfIteratorHelper(route),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var key = _step.value;
        data = data[key];
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
    var fieldData = data;
    var logError = function logError(error) {
      console.error(error, route.join('/') + '+' + field, value, fieldData);
    };

    // 初始化动作修改路径

    switch (type) {
      case 'create':
        if (!field) {
          logError('未指定field');
          return;
        }
        if (fieldData instanceof Array) {
          // 给array push 一个新东西
          var index = parseInt(field);
          fieldData[index] = value;
        } else if (fieldData instanceof Object) {
          // 给对象创建新的属性
          if (!fieldData.hasOwnProperty(field)) {
            fieldData[field] = value;
          }
        } else {
          logError('对非对象/数组的错误创建请求');
        }
        break;
      case 'change':
        // change 是对非对象值的改变
        if (field === undefined) {
          s.data = cloneDeep(value);
        } else if (fieldData instanceof Array || fieldData instanceof Object) fieldData[field] = cloneDeep(value);else logError('对非对象/数组的字段修改请求');
        break;
      case 'delete':
        if (!field) return;
        if (fieldData instanceof Array) {
          // 注意数组删除，后面元素是要前移的
          var _index = parseInt(field);
          console.log(_index, fieldData[_index]);
          remove(fieldData, function (value, i) {
            return i === _index;
          });
        } else if (fieldData instanceof Object) delete fieldData[field];else {
          logError('对非对象/数组的字段删除请求');
        }
        break;
      case 'rename':
        if (!field || !value || field === value) break;
        if (fieldData instanceof Object) {
          // todo: 严查value类型
          if (!fieldData.hasOwnProperty(value)) {
            fieldData[value] = fieldData[field];
            delete fieldData[field];
          }
        } else {
          logError('对非对象的字段重命名请求');
        }
        break;
      case 'moveup':
      case 'movedown':
        if (fieldData instanceof Array) {
          if (!field) return;
          var _index2 = parseInt(field);
          var swapIndex = type === 'moveup' ? _index2 - 1 : _index2 + 1;
          if (swapIndex >= 0 && swapIndex < fieldData.length) {
            var temp = fieldData[_index2];
            fieldData[_index2] = fieldData[swapIndex];
            fieldData[swapIndex] = temp;
          } else {
            logError('数组项越界移动');
          }
        } else logError('对非数组的移动请求');
        break;
      default:
        logError('错误的动作请求');
    }
    // 重新验证
    var reValidateValue = type === 'change' && field !== undefined ? fieldData[field] : fieldData;
    var reValidatePath = type === 'change' && field !== undefined ? route.concat(field) : route;
    reValidate(reValidateValue, reValidatePath);
    return;
  }, {
    data: {},
    dataErrors: {}
  }), {
    undoType: 'undo',
    redoType: 'redo',
    limit: 35
  });
};